@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2017-2018  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2017-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@
 #define _GNU_SOURCE
 #include <stdio.h>
 #include <string.h>
+#include <getopt.h>
 #include <ell/ell.h>
 #include <readline/readline.h>
 
@@ -34,6 +35,7 @@
 #include "display.h"
 
 static struct l_queue *command_families;
+static struct l_queue *command_options;
 static int exit_status;
 static bool interactive_mode;
 static struct command_noninteractive {
@@ -41,6 +43,45 @@ static struct command_noninteractive {
 	int argc;
 } command_noninteractive;
 
+struct command_option {
+	const char *name;
+	char *value;
+};
+
+static void command_options_destroy(void *data)
+{
+	struct command_option *option = data;
+
+	l_free(option->value);
+	l_free(option);
+}
+
+bool command_option_get(const char *name, const char **value_out)
+{
+	const struct l_queue_entry *entry;
+
+	for (entry = l_queue_get_entries(command_options); entry;
+							entry = entry->next) {
+		const struct command_option *option = entry->data;
+
+		if (strcmp(option->name, name))
+			continue;
+
+		if (value_out)
+			*value_out = option->value;
+
+		return true;
+	}
+
+	return false;
+}
+
+bool command_needs_no_agent(void)
+{
+	return command_option_get(COMMAND_OPTION_DONTASK, NULL) &&
+					(l_queue_length(command_options) == 1);
+}
+
 static enum cmd_status cmd_version(const char *entity,
 						char **argv, int argc)
 {
@@ -466,6 +507,20 @@ static void list_commands(const char *command_family,
 	}
 }
 
+static void list_cmd_options(void)
+{
+	display(MARGIN "--%-*s%s\n", 48, COMMAND_OPTION_USERNAME,
+					"Provide username");
+	display(MARGIN "--%-*s%s\n", 48, COMMAND_OPTION_PASSWORD,
+					"Provide password");
+	display(MARGIN "--%-*s%s\n", 48, COMMAND_OPTION_PASSPHRASE,
+					"Provide passphrase");
+	display(MARGIN "--%-*s%s\n", 48, COMMAND_OPTION_DONTASK,
+					"Don’t ask for missing\n"
+					"\t\t\t\t\t\t    credentials");
+	display(MARGIN "--%-*s%s\n", 48, "help", "Display help");
+}
+
 static void list_cmd_families(void)
 {
 	const struct l_queue_entry *entry;
@@ -479,6 +534,32 @@ static void list_cmd_families(void)
 	}
 }
 
+static void command_display_help(void)
+{
+	display("\n");
+	display_table_header("iwctl version " VERSION, MARGIN "%-*s",
+								5, "Usage");
+	display(MARGIN "iwctl [--options] [commands]\n");
+	display_table_footer();
+
+	display_table_header("Available options", MARGIN "%-*s%-*s",
+					50, "Options", 28, "Description");
+	list_cmd_options();
+	display_table_footer();
+
+	display_table_header("Available commands", MARGIN "%-*s%-*s",
+					50, "Commands", 28, "Description");
+	list_cmd_families();
+	display_table_footer();
+
+	if (!interactive_mode)
+		return;
+
+	display("\nMiscellaneous:\n");
+
+	list_commands(NULL, misc_commands);
+}
+
 static bool command_match_misc_commands(char **argv, int argc)
 {
 	if (match_cmd(NULL, argv[0], argv + 1, argc - 1, misc_commands))
@@ -487,17 +568,7 @@ static bool command_match_misc_commands(char **argv, int argc)
 	if (strcmp(argv[0], "help"))
 		return false;
 
-	display_table_header("Available commands", MARGIN "%-*s%-*s",
-					50, "Commands", 28, "Description");
-
-	list_cmd_families();
-
-	if (!interactive_mode)
-		return true;
-
-	display("\nMiscellaneous:\n");
-
-	list_commands(NULL, misc_commands);
+	command_display_help();
 
 	return true;
 }
@@ -555,6 +626,21 @@ int command_get_exit_status(void)
 	return exit_status;
 }
 
+void command_reset_default_entities(void)
+{
+	const struct l_queue_entry *entry;
+
+	for (entry = l_queue_get_entries(command_families); entry;
+							entry = entry->next) {
+		struct command_family *family = entry->data;
+
+		if (!family->reset_default_entity)
+			continue;
+
+		family->reset_default_entity();
+	}
+}
+
 void command_family_register(const struct command_family *family)
 {
 	l_queue_push_tail(command_families, (void *) family);
@@ -565,14 +651,25 @@ void command_family_unregister(const struct command_family *family)
 	l_queue_remove(command_families, (void *) family);
 }
 
+static const struct option command_opts[] = {
+	{ COMMAND_OPTION_USERNAME,	required_argument, NULL, 'u' },
+	{ COMMAND_OPTION_PASSWORD,	required_argument, NULL, 'p' },
+	{ COMMAND_OPTION_PASSPHRASE,	required_argument, NULL, 'P' },
+	{ COMMAND_OPTION_DONTASK,	no_argument,	   NULL, 'd' },
+	{ "help",			no_argument,	   NULL, 'h' },
+	{ }
+};
+
 extern struct command_family_desc __start___command[];
 extern struct command_family_desc __stop___command[];
 
 bool command_init(char **argv, int argc)
 {
 	struct command_family_desc *desc;
+	int opt;
 
 	command_families = l_queue_new();
+	command_options = l_queue_new();
 
 	for (desc = __start___command; desc < __stop___command; desc++) {
 		if (!desc->init)
@@ -581,13 +678,69 @@ bool command_init(char **argv, int argc)
 		desc->init();
 	}
 
-	if (argc < 2) {
-		interactive_mode = true;
-		return true;
+	for (;;) {
+		struct command_option *option;
+
+		opt = getopt_long(argc, argv, "u:p:P:dh", command_opts, NULL);
+
+		switch (opt) {
+		case 'u':
+			option = l_new(struct command_option, 1);
+			option->name = COMMAND_OPTION_USERNAME;
+			option->value = l_strdup(optarg);
+
+			l_queue_push_tail(command_options, option);
+
+			break;
+		case 'p':
+			option = l_new(struct command_option, 1);
+			option->name = COMMAND_OPTION_PASSWORD;
+			option->value = l_strdup(optarg);
+
+			l_queue_push_tail(command_options, option);
+
+			break;
+		case 'P':
+			option = l_new(struct command_option, 1);
+			option->name = COMMAND_OPTION_PASSPHRASE;
+			option->value = l_strdup(optarg);
+
+			l_queue_push_tail(command_options, option);
+
+			break;
+		case 'd':
+			option = l_new(struct command_option, 1);
+			option->name = COMMAND_OPTION_DONTASK;
+
+			l_queue_push_tail(command_options, option);
+
+			break;
+		case 'h':
+			command_display_help();
+
+			l_main_quit();
+
+			return true;
+		case -1:
+			goto options_parsed;
+		case '?':
+			exit_status = EXIT_FAILURE;
+
+			return true;
+		}
 	}
 
-	command_noninteractive.argv = argv + 1;
-	command_noninteractive.argc = argc - 1;
+options_parsed:
+	argv += optind;
+	argc -= optind;
+
+	if (argc < 2) {
+		interactive_mode = true;
+		return false;
+	}
+
+	command_noninteractive.argv = argv;
+	command_noninteractive.argc = argc;
 
 	return false;
 }
@@ -605,4 +758,7 @@ void command_exit(void)
 
 	l_queue_destroy(command_families, NULL);
 	command_families = NULL;
+
+	l_queue_destroy(command_options, command_options_destroy);
+	command_options = NULL;
 }
