@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2015-2018  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2015-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -37,12 +37,14 @@
 
 #include "linux/nl80211.h"
 #include "src/iwd.h"
+#include "src/module.h"
 #include "src/wiphy.h"
 #include "src/ie.h"
 #include "src/common.h"
 #include "src/network.h"
 #include "src/knownnetworks.h"
 #include "src/nl80211cmd.h"
+#include "src/nl80211util.h"
 #include "src/util.h"
 #include "src/scan.h"
 
@@ -264,46 +266,27 @@ static void scan_build_attr_scan_frequencies(struct l_genl_msg *msg,
 	l_genl_msg_leave_nested(msg);
 }
 
-static bool scan_mac_address_randomization_is_disabled(void)
-{
-	const struct l_settings *config = iwd_get_config();
-	bool disabled;
-
-	if (!l_settings_get_bool(config, "Scan",
-					"disable_mac_address_randomization",
-					&disabled))
-		return false;
-
-	return disabled;
-}
-
-static struct l_genl_msg *scan_build_cmd(struct scan_context *sc,
-					bool ignore_flush_flag, bool is_passive,
+static void scan_build_attr_ie(struct l_genl_msg *msg,
+					struct scan_context *sc,
 					const struct scan_parameters *params)
 {
-	struct l_genl_msg *msg;
-	uint32_t flags = 0;
 	struct iovec iov[3];
 	unsigned int iov_elems = 0;
 	const uint8_t *ext_capa;
 	uint8_t interworking[3];
 
-	msg = l_genl_msg_new(NL80211_CMD_TRIGGER_SCAN);
-
-	l_genl_msg_append_attr(msg, NL80211_ATTR_WDEV, 8, &sc->wdev_id);
-
 	ext_capa = wiphy_get_extended_capabilities(sc->wiphy,
 							NL80211_IFTYPE_STATION);
 	/*
-	 * XXX: If adding IE's here ensure that ordering is not broken for
-	 * probe requests (IEEE-2016 Table 9-33).
+	 * If adding IE's here ensure that ordering is not broken for
+	 * probe requests (IEEE Std 802.11-2016 Table 9-33).
 	 */
 	/* Order 9 - Extended Capabilities */
 	iov[iov_elems].iov_base = (void *) ext_capa;
 	iov[iov_elems].iov_len = ext_capa[1] + 2;
 	iov_elems++;
 
-	if (util_is_bit_set(ext_capa[3], 7)) {
+	if (util_is_bit_set(ext_capa[2 + 3], 7)) {
 		/* Order 12 - Interworking */
 		interworking[0] = IE_TYPE_INTERWORKING;
 		interworking[1] = 1;
@@ -323,6 +306,34 @@ static struct l_genl_msg *scan_build_cmd(struct scan_context *sc,
 	}
 
 	l_genl_msg_append_attrv(msg, NL80211_ATTR_IE, iov, iov_elems);
+}
+
+static bool scan_mac_address_randomization_is_disabled(void)
+{
+	const struct l_settings *config = iwd_get_config();
+	bool disabled;
+
+	if (!l_settings_get_bool(config, "Scan",
+					"DisableMacAddressRandomization",
+					&disabled))
+		return false;
+
+	return disabled;
+}
+
+static struct l_genl_msg *scan_build_cmd(struct scan_context *sc,
+					bool ignore_flush_flag, bool is_passive,
+					const struct scan_parameters *params)
+{
+	struct l_genl_msg *msg;
+	uint32_t flags = 0;
+
+	msg = l_genl_msg_new(NL80211_CMD_TRIGGER_SCAN);
+
+	l_genl_msg_append_attr(msg, NL80211_ATTR_WDEV, 8, &sc->wdev_id);
+
+	if (wiphy_get_max_scan_ie_len(sc->wiphy))
+		scan_build_attr_ie(msg, sc, params);
 
 	if (params->freqs)
 		scan_build_attr_scan_frequencies(msg, params->freqs);
@@ -346,10 +357,46 @@ static struct l_genl_msg *scan_build_cmd(struct scan_context *sc,
 	if (flags)
 		l_genl_msg_append_attr(msg, NL80211_ATTR_SCAN_FLAGS, 4, &flags);
 
-	if (params->no_cck_rates)
+	if (params->no_cck_rates) {
+		static const uint8_t b_rates[] = { 2, 4, 11, 22 };
+		uint8_t *scan_rates;
+		const uint8_t *supported;
+		unsigned int num_supported;
+		unsigned int count;
+		unsigned int i;
+
 		l_genl_msg_append_attr(msg, NL80211_ATTR_TX_NO_CCK_RATE, 0,
 					NULL);
 
+		/*
+		 * Assume if we're sending the probe requests at OFDM bit
+		 * rates we don't want to advertise support for 802.11b rates.
+		 */
+		if (L_WARN_ON(!(supported = wiphy_get_supported_rates(sc->wiphy,
+							NL80211_BAND_2GHZ,
+							&num_supported))))
+			goto done;
+
+		scan_rates = l_malloc(num_supported);
+
+		for (count = 0, i = 0; i < num_supported; i++)
+			if (!memchr(b_rates, supported[i],
+						L_ARRAY_SIZE(b_rates)))
+				scan_rates[count++] = supported[i];
+
+		if (L_WARN_ON(!count)) {
+			l_free(scan_rates);
+			goto done;
+		}
+
+		l_genl_msg_enter_nested(msg, NL80211_ATTR_SCAN_SUPP_RATES);
+		l_genl_msg_append_attr(msg, NL80211_BAND_2GHZ,
+							count, scan_rates);
+		l_genl_msg_leave_nested(msg);
+		l_free(scan_rates);
+	}
+
+done:
 	return msg;
 }
 
@@ -640,7 +687,7 @@ static bool scan_periodic_is_disabled(void)
 	const struct l_settings *config = iwd_get_config();
 	bool disabled;
 
-	if (!l_settings_get_bool(config, "Scan", "disable_periodic_scan",
+	if (!l_settings_get_bool(config, "Scan", "DisablePeriodicScan",
 								&disabled))
 		return false;
 
@@ -1359,10 +1406,8 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 	uint16_t type, len;
 	const void *data;
 	uint8_t cmd;
-	uint64_t uninitialized_var(attr_wdev_id);
-	bool have_wdev_id;
-	uint32_t uninitialized_var(attr_wiphy);
-	bool have_wiphy;
+	uint64_t wdev_id;
+	uint32_t wiphy_id;
 	struct scan_context *sc;
 	bool active_scan = false;
 
@@ -1370,49 +1415,26 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 
 	l_debug("Scan notification %s(%u)", nl80211cmd_to_string(cmd), cmd);
 
+	if (nl80211_parse_attrs(msg, NL80211_ATTR_WDEV, &wdev_id,
+					NL80211_ATTR_WIPHY, &wiphy_id,
+					NL80211_ATTR_UNSPEC) < 0)
+		return;
+
+	sc = l_queue_find(scan_contexts, scan_context_match, &wdev_id);
+	if (!sc)
+		return;
+
 	if (!l_genl_attr_init(&attr, msg))
 		return;
 
 	while (l_genl_attr_next(&attr, &type, &len, &data)) {
 		switch (type) {
-		case NL80211_ATTR_WIPHY:
-			if (len != sizeof(uint32_t)) {
-				l_warn("Invalid wiphy attribute");
-				return;
-			}
-
-			have_wiphy = true;
-			attr_wiphy = *((uint32_t *) data);
-			break;
-		case NL80211_ATTR_WDEV:
-			if (len != sizeof(uint64_t)) {
-				l_warn("Invalid wdev index attribute");
-				return;
-			}
-
-			have_wdev_id = true;
-			attr_wdev_id = *((uint64_t *) data);
-			break;
 		case NL80211_ATTR_SCAN_SSIDS:
 			active_scan = true;
 			break;
 		}
 	}
 
-	if (!have_wiphy) {
-		l_warn("Scan results do not contain wiphy attribute");
-		return;
-	}
-
-	if (!have_wdev_id) {
-		l_warn("Scan results do not contain wdev attribute");
-		return;
-	}
-
-	sc = l_queue_find(scan_contexts, scan_context_match, &attr_wdev_id);
-	if (!sc)
-		return;
-
 	switch (cmd) {
 	case NL80211_CMD_NEW_SCAN_RESULTS:
 	{
@@ -1547,7 +1569,7 @@ uint8_t scan_freq_to_channel(uint32_t freq, enum scan_band *out_band)
 	}
 
 	if (freq >= 5005 && freq < 5900) {
-		if (channel % 5)
+		if (freq % 5)
 			return 0;
 
 		channel = (freq - 5000) / 5;
@@ -1559,7 +1581,7 @@ uint8_t scan_freq_to_channel(uint32_t freq, enum scan_band *out_band)
 	}
 
 	if (freq >= 4905 && freq < 5000) {
-		if (channel % 5)
+		if (freq % 5)
 			return 0;
 
 		channel = (freq - 4000) / 5;
@@ -1952,7 +1974,7 @@ static int scan_init(void)
 
 	scan_contexts = l_queue_new();
 
-	if (!l_settings_get_double(config, "Rank", "rank_5g_factor",
+	if (!l_settings_get_double(config, "Rank", "BandModifier5Ghz",
 					&RANK_5G_FACTOR))
 		RANK_5G_FACTOR = 1.0;
 
